# build a function to create the tables based on the window size
def makeWindow(ws, ts, dIn,dOut):
    for i in range(ws-1,len(ts)):
        for j in range(ws-1,0,-1):
            dIn.append(ts[i-j])
        dOut.append(ts[i])



# build a function sending data to R and receiving results
def estimateNN_and_predict(plt,cn,description,ws,hN,nF,dIn,dOu):
    
    cn.r.In=dIn
    cn.r.Ou=dOu
    cn.r.ws=ws
    cn.r.hN=hN
    cn.r.nF=nF
    cn.r.description=description
    # preparing data, we will limit the rows to len(dOu)-nF
    cn.r.useRows=len(dOu)-nF
    cn.r('useRowsCols=useRows*(ws-1)')

    cn.r('In = as.numeric(In)') # data are sent as characters, rebuild numbers
    cn.r('Ou = as.numeric(Ou)')
    #cn.r('print(In)')
    #cn.r('print(Ou)')
    
    cn.r('InM=matrix(In[1:useRowsCols],ncol=ws-1,byrow=TRUE)')
    #cn.r('print(InM)')

    # scaling
    cn.r('adjust   = function(x,xmin,xmax) {(x-xmin)/(xmax-xmin)}')
    #cn.r('unadjust = function(x,xmin,xmax) {x*(xmax-xmin)+xmin}')

    cn.r('InMA = adjust(InM, min(In), max(In))') # min/max on the whole set of 
                                                 # data, not only of the used
                                                 # used subset
    #cn.r('print(InMA)')
    cn.r('OuA = adjust(Ou[1:useRows], min(Ou), max(Ou))') # min/max se above
    #cn.r('print(OuA)')

    # calculating the NNs (10 trials each)
    cn.r.err=100
    errs=[]
    for i in range(10):
        cn.r('myNet=nnet(x=InMA, y=OuA, size=hN, maxit=1000)')

        # saving the best NN   
        cn.r('er=sum(myNet$residuals**2)')
        cn.r('if(er<err)bestNet=myNet')
        cn.r('if(er<err)err=er')
        errs.append(cn.r.er)
    print ("Min error", cn.r.err,"for dataset '"+description+"' in trial #", errs.index(min(errs)),\
           "counting from 0.")

    # view errors
    plt.figure()
    ind=range(len(errs))
    plt.bar(ind, errs)
    plt.ylabel('sum ot the squared errors')
    plt.title(description+" (absolute values)")
    plt.show()


    # to analize the content of bestNet or of other complex object in R, use str() 
            
    # to have a look to the results as fitted values vs. original one, in Python
    #fit=cn.r('bestNet$fitted.values')
    #print (cn.r.OuA)
    #print (fit)    
    
    # plot orignal values vs. fitted ones (scaled)
    cn.r('fit=ts(bestNet$fitted.values)') # fitted values
    cn.r('OuA2 = ts(adjust(Ou, min(Ou), max(Ou)))') # orignal ones (all)
    
    # forecasting with actual input data
    cn.r('useRowsCols1=useRowsCols+1')
    cn.r.endRowsCols=len(dOu)*(ws-1)
    #cn.r('print(useRowsCols1)')
    #cn.r('print(endRowsCols)')
    cn.r('InMLastPart=matrix(In[useRowsCols1:endRowsCols],ncol=ws-1,byrow=TRUE)')
    cn.r('InMALastPart = adjust(InMLastPart, min(In), max(In))')
    cn.r('pred=predict(bestNet,InMALastPart)')
    # adding in first position the last actual value
    cn.r('pred_plus=c(OuA[length(OuA)],pred)')
    #pred=cn.r.pred
    #print (pred)
    cn.r.lastRows=len(dOu)-nF    
    cn.r('pred_plus=ts(pred_plus,start=lastRows)')
    
    # forecasting with forecasted input data
    #cn.r('print(InMALastPart)')
    #cn.r('print(pred)')
    cn.r('v=InMALastPart[1,]')
    for i in range(1,nF+1):
        cn.r.i=i      
        #cn.r('print(v)')
        cn.r('pred[i]=predict(bestNet,v)')
        #cn.r('print(pred)')
        for j in range(1,cn.r('length(v)')):
            cn.r.j=j
            cn.r('v[j]=v[j+1]')
        cn.r('v[length(v)]=pred[i]')
    # adding in first position the last actual value
    cn.r('pred2_plus=c(OuA[length(OuA)],pred)')
    cn.r('pred2_plus=ts(pred2_plus,start=lastRows)')
    
    # AR section, as an example
    # comment to eliminate (the functions manages internally R windows)
    AR(cn)
    # on our data, autocorrelations and partial autocorrelations show an AR(1) 
    # process on absolute values (autocorrelations smoothly decreasing and only
    # one signinificant partial autocorrelation). On returns we have no evident
    # aucorrelation process
    
    # In both cases we apply here, as a calculation exercise, a AR estimate with
    # an unbounded number of lags
    
    # We can extend the content of the AR function to MA or ARIMA estimates
    # For a sintetic note look at 
    # http://terna.to.it/materiale/time_ser_2008_09.doc
    # (sorry, in Italian)
    
    
    
    
    # to save the graphic, uncomment and unindent a proper location for 
    # the files, e.g.
    # cn.r('setwd("/home/pt/Desktop/series/")')
    # cn.r('setwd("/Users/macbookpro/Desktop/timeSeriesNNs/")')
    # to save a png or ps file (uncomment and unindent one of the next two rows)
    # cn.r('png(paste(description,"_plot.png",sep=""))')
    # cn.r('postscript(paste(description,"_plot.ps",sep=""),horizontal=FALSE)')
    cn.r('x11()')
    cn.r('ts.plot(OuA2,fit,pred_plus,pred2_plus,ar_pred_plus,gpars=list(main=description,xlab="years",ylab="fuels",lty=c(1,2,3,4,5),col=c("black","green","red","blue","brown")))')
    
    print ("black = actual data")
    print ("green = fitted data")
    print ("red   = predicted data, but based on actual ones")
    print ("blue  = predicted data, based recursively on forecasted ones")
    print ("brown = AR predictions")


# build a function making AR estimates via R
def AR(cn):
    #cn.r('ar_series=OuA')
    cn.r('x11()')
    cn.r('par(mfrow=c(1,2))')
    cn.r('print(OuA)')
    
    cn.r('acf(OuA,type="cor")')    #   autocorrelation
    cn.r('acf(OuA,type="par")')    #   partial autocorrelatio
    input("enter to continue") 
    cn.r('dev.off()')
    
    # ar estimate
    cn.r('ar.ar=ar(OuA)')
    cn.r('print(ar.ar)')           #   look at the R window
    
    cn.r('ar.pred=predict(ar.ar,OuA,n.ahead=nF)')
    #cn.r('print(ar_pred)')
    cn.r('ar_pred_plus=ts(c(OuA[length(OuA)],ar.pred$pred),start=lastRows)')
    cn.r('print(ar_pred_plus)')
    return
